﻿using System;
using UnityEngine;
using Windows.Kinect;

public class CoordinateMapperView : MonoBehaviour
{
	private ComputeBuffer DepthBuffer,
                          BodyIndexBuffer;

	private DepthSpacePoint[] DepthPoints;
	private byte[] BodyIndexPoints;
	
	void Start ()
	{
        CoordinateMapperManager CoordinateMapperManager = GameObject.Find("Managers").GetComponent<CoordinateMapperManager>();
		
        GetComponent<Renderer>().material.SetTexture("_MainTex", CoordinateMapperManager.ColorTexture);

        Func<Array, Array, Action<string, ComputeBuffer, int>> setBuffer = (srcPoints, dstPoints) =>
        {
            dstPoints = srcPoints;
            return (name, buffer, stride) =>
            {
                buffer = new ComputeBuffer(dstPoints.Length, stride);
                GetComponent<Renderer>().material.SetBuffer(name, buffer);
            };
        };

        setBuffer(CoordinateMapperManager.DepthCoordinates, DepthPoints)
            ("depthCoordinates", DepthBuffer, sizeof(float) * 2);

        setBuffer(CoordinateMapperManager.BodyIndexBuffer, BodyIndexPoints)
            ("BodyIndexBuffer", BodyIndexBuffer, sizeof(float));
    }

	void Update()
	{
		//TODO: fix perf on this call.
		DepthBuffer.SetData(DepthPoints);
        
		BodyIndexBuffer.SetData(System.Array.ConvertAll(BodyIndexPoints, System.Convert.ToSingle));
	}
	
	void OnDisable() 
	{
        DepthBuffer.Release();   
        BodyIndexBuffer.Release();

        DepthPoints = null;
        BodyIndexPoints = null;
    }
}
